@@ -1,4 +1,4 @@
-from typing import List, Type
+from typing import Dict, Type
 
 from django.db import models
 from django.db.models.fields import Field
@@ -7,6 +7,14 @@ from ..helpers import build_variations
 from ..models.base import VersatileImageResourceMixin
 from ..models.collection import CollectionBase
 from ..models.fields.collection import CollectionItem
+from ..variations import PaperVariation
+
+
+def get_model_name(model: Type[models.Model]) -> str:
+    return "{}.{}".format(
+        model._meta.app_label,
+        model.__name__
+    )
 
 
 def is_variations_allowed(model: Type[models.Model]) -> bool:
@@ -23,20 +31,20 @@ def is_collection(model: Type[models.Model]) -> bool:
     return issubclass(model, CollectionBase)
 
 
-def get_field_variations(field: Field) -> List[str]:
+def get_field_variations(field: Field) -> Dict[str, PaperVariation]:
     """
-    Получение списка имен вариаций для поля.
+    Получение списка вариаций для поля.
     """
-    return list(
-        field.variations.keys()
-    )
+    variation_config = getattr(field, "variations", {}).copy()
+    return build_variations(variation_config)
 
 
-def get_collection_variations(collection_cls: Type[CollectionBase], item_type_field: CollectionItem) -> List[str]:
+def get_collection_variations(
+    collection_cls: Type[CollectionBase], item_type_field: CollectionItem
+) -> Dict[str, PaperVariation]:
     """
-    Получение списка имен вариаций для коллекции.
+    Получение списка вариаций для коллекции.
     """
     image_model = item_type_field.model
     variation_config = image_model.get_variation_config(collection_cls, item_type_field)
-    variations = build_variations(variation_config)
-    return list(variations.keys())
+    return build_variations(variation_config)
